package org.halfdog.io.parport;

import java.util.ArrayList;

public class GenericSequence implements Sequence {
// steps belonging to this sequence
  private ArrayList		sequence;

// cooked format to allow faster access
  private byte			sequence_bytes[];

// the number of reads in the sequence
  private int			reads;
  private boolean		changed;



  protected GenericSequence() {
    sequence=new ArrayList();
    clear();
  }



  public void clear() {
    sequence.clear();
    sequence_bytes=null;
    changed=false;
    reads=0;
  }




/** Add a new step to the end of a sequence.
 *  @return the number of the step in the sequence
*/
  public synchronized int addStep(int command, int value, int wait) {
    int i, vals[]=new int[3];
    vals[0]=command;
    vals[1]=value;
    vals[2]=wait;

    sequence.add(vals);
    changed=true;
    return(sequence.size()-1);
  }


  protected byte[] getSequence() {
    if(changed) genSequence();
    return(sequence_bytes);
  }


  protected int getReadCount() {
    if(changed) genSequence();
    return(reads);
  }

/** Build the byte oriented sequence data for the stored
 *  commands.
 */
  private synchronized void genSequence() {
    if((sequence_bytes==null)||
       (sequence_bytes.length!=((sequence.size()<<2)+4))) {
      sequence_bytes=new byte[sequence.size()<<2];
    }
    reads=0;

    int n=0;
    for(n=0; n<4; n++) sequence_bytes[n]=(byte)(sequence.size()>>(n<<3));

    for(int i=0; i<sequence.size(); i++) {
      int vals[]=(int[])sequence.get(i);
      sequence_bytes[n]=(byte)vals[0];
      n++;

      switch(vals[0]) {
        case ParPort.PPORT_RDATA:
        case ParPort.PPORT_RSTATUS:
          reads++;
        case ParPort.PPORT_WCTRL:
          sequence_bytes[n]=(byte)(vals[1]);
          break;
        case ParPort.PPORT_WDATA:
          sequence_bytes[n]=(byte)vals[1];
          break;
        default:
          throw(new IllegalArgumentException("Illegal command "+vals[0]+
                                             " in sequence at position "+i));
      }
      n++;
      sequence_bytes[n]=(byte)(vals[2]); n++;
      sequence_bytes[n]=(byte)((vals[2]>>8)&0xFF);
      n++;
    }
    changed=false;
  }

}
